#ifndef	INSTRUCTION_H_
#define	INSTRUCTION_H_

#include "visitor.h"

// Base class for concrete instructions
class Instruction
{
	string name;

public:
	Instruction(const char *name)
	{
		this->name = name;
	}
	virtual ~Instruction() {};

	string GetName(void)
	{
		return name;
	}
	
	virtual void Accept(Visitor *v) = 0;
};

// instruction "If"
class If : public Instruction
{
public:
	If(const char *name) : Instruction(name) {}
	void Accept(Visitor *v)
	{
		v->Visit(this);
	}
};

// instruction "Assign"
class Assign : public Instruction
{
public:
	Assign(const char *name) : Instruction(name) {}
	void Accept(Visitor *v)
	{
		v->Visit(this);
	}
};

// instruction "Repeat"
class Repeat : public Instruction
{
public:
	Repeat(const char *name) : Instruction(name) {}
	void Accept(Visitor *v)
	{
		v->Visit(this);
	}
};

// instruction "Read"
class Read : public Instruction
{
public:
	Read(const char *name) : Instruction(name) {} 
	void Accept(Visitor *v)
	{
		v->Visit(this);
	}
};

// instruction "Write"
class Write : public Instruction
{
public:
	Write(const char *name) : Instruction(name) {}
	void Accept(Visitor *v)
	{
		v->Visit(this);
	}
};

#endif

